<?php
///////////////////////////////////////////////////////////////////////////
// includes
require_once 'lib/default_dune_plugin_fw.php';
require_once 'lib/default_dune_plugin.php';
require_once 'lib/utils.php';
require_once 'lib/i18n.php';
require_once '24tvApi.php';
require_once 'lib/user_input_handler_registry.php';
require_once 'lib/media_url.php';
require_once 'lib/action_factory.php';


///////////////////////////////////////////////////////////////////////////
// plugin class name
DefaultDunePluginFw::$plugin_class_name = '_24tvPlugin';

///////////////////////////////////////////////////////////////////////////
// plugin class
class _24tvPlugin extends DefaultDunePlugin implements UserInputHandler {
    
    const ID = 'tv_channel_list';

    // menu
    private static $menu = 'menu';

    // online menu itme
    private static $online = 'online';

    // action on folder open
    private static $open_folder_action = array (
        GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID
    );

    // action on video item play
    private static $vod_play_action = array (
        GuiAction::handler_string_id => PLUGIN_TV_PLAY_ACTION_ID
    );

    // API instance
    private $api;

    // current_channel_id
    private $current_channel_id;

    // current_news_item_ind
    private $current_news_item_index;

    ///////////////////////////////////////////////////////////////////////
    // construct
    public function __construct() {
        $this->api = new _24tvApi();
        
        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }
    
    public function get_handler_id() { return self::ID; }
    
    ///////////////////////////////////////////////////////////////////////
    // Returns current folder content representation data for the plugin menu folder identified by $media_url.
    public function get_folder_view($media_url, &$plugin_cookies) {
        
        //hd_print("======= get_folder_view, media_url=" . $media_url);
        

        if( $plugin_cookies->{'device_uniqid'} ) {
            $device_uniqid = $plugin_cookies->{'device_uniqid'};
        } else {
            $device_uniqid = uniqid();
            $plugin_cookies->{'device_uniqid'} = $device_uniqid;
        }
        _24tvApi::$piwikDeviceId = $device_uniqid;

        
        if ($media_url === self::$menu)
            return $this->get_main_folder_view($media_url, $plugin_cookies);

        if ($media_url === _24tvApi::$favorites_url)
            return $this->get_category_folder_view($media_url, $plugin_cookies);

        if ($media_url ===  self::$online)
            // TODO: return online
            return $this->get_main_folder_view($media_url, $plugin_cookies);

        return $this->get_category_folder_view($media_url, $plugin_cookies);

    }

    ///////////////////////////////////////////////////////////////////////
    // Returns the specified chunk of folder element for the folder identified by $media_url
    public function get_regular_folder_items($media_url, $from_ndx, &$plugin_cookies) {

        //hd_print("======= get_regular_folder_items, media_url=" . $media_url . " ======= from_ndx=" . $from_ndx);

        if ($media_url === self::$menu) {
            return $this->get_root_menu_folders();
        }

        // TODO: favorites
        if ($media_url === _24tvApi::$favorites_url) {
            return $this->get_favorites_news(false, $plugin_cookies);
        }

        return $this->get_category_news($media_url);

        /*
        if ($user_data->section_id === 'top')
            return $this->get_movie_list_folder_items(
                array ('cmd' => 'top', 'order' => $user_data->sort_order));
        }
        else if ($user_data->section_id === 'movie_list')
        {
            return $this->get_movie_list_folder_items(
                array ('cmd' => 'videolist', 'data' => $user_data->category_id));
        }
        */

        throw new Exception('Unexpected media_url');
    }

    public function get_tv_info($media_url, &$plugin_cookies) {
        //hd_print("======= get_tv_info, media_url = $media_url");
        // if play button pressed in the root menu
        if (is_numeric($media_url)) {
            $channel_id = $media_url;
            $this->current_news_item_index = 0;
            $timestamp = -1;
        } else {
            $media_str = null; $user_data = null;
            HD::decode_user_data($media_url, $media_str, $user_data);
            $this->current_news_item_index = $user_data->news_item_index;
            $channel_id = $user_data->channel_id;
            $timestamp = $user_data->timestamp;
            $group_id = null;
            
        }
        if($channel_id == _24tvApi::$tv24_channel_id) {
            $group_id = _24tvApi::$categories_group_id;
        } else {
            $group_id = _24tvApi::$categories_group_id;
        }
/*
        $fav_channel_ids = null;
        if ($this->api->is_favorites_supported())
            $fav_channel_ids = $this->api->get_fav_channel_ids($plugin_cookies);
*/
        $this->current_channel_id = $channel_id;
        //hd_print("======= get_tv_info, group_id / channel_id / timestamp = $group_id / $channel_id / $timestamp");
        
        return array (
            PluginTvInfo::show_group_channels_only => true,
            PluginTvInfo::groups => $this->api->getGroups(), 
            PluginTvInfo::channels => $this->api->getChannels(),
            //PluginTvInfo::show_group_channels_only => false,
            // set selections in EPG
            PluginTvInfo::initial_group_id => $group_id,

            PluginTvInfo::initial_channel_id => $channel_id,
            PluginTvInfo::initial_archive_tm => $timestamp,
/*
            PluginTvInfo::favorites_supported => $this->api->is_favorites_supported(),
            PluginTvInfo::favorites_icon_url => strval($this->api->get_favorites_icon_url()),
            PluginTvInfo::initial_is_favorite => $this->api->is_favorites_supported(),
            PluginTvInfo::favorite_channel_ids => $fav_channel_ids,            
*/          
            PluginTvInfo::ip_address_required => true
        );

    }    
    
    ///////////////////////////////////////////////////////////////////////
    // &$user_input,
    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies) {
        //hd_print("======= get_tv_playback_url=$channel_id, $archive_ts, $protect_code");
        if( $channel_id == 0 && $channel_id != _24tvApi::$favorites_url ) {
            $this->api->request("http://piwik.luxnet.ua/piwik.php?idsite=" . _24tvApi::$piwikSiteId . "&rec=1&apiv=1&idgoal=" . _24tvApi::$piwikPlayOnlineIdGoal . "&_id=" . _24tvApi::$piwikDeviceId);
        } else {
            $this->api->request("http://piwik.luxnet.ua/piwik.php?idsite=" . _24tvApi::$piwikSiteId . "&rec=1&apiv=1&idgoal=" . _24tvApi::$piwikPlayVideoIdGoal . "&_id=" . _24tvApi::$piwikDeviceId);
        }
        if($channel_id != $this->current_channel_id) {
            // channel changed through transparent menu
            $this->current_channel_id = $channel_id;
            $this->current_news_item_index = 0; // start playing first news item from the category
        }
        // array of news for selected channel
        if( $channel_id == _24tvApi::$favorites_url ) {
            $currentCategoryNews = $this->get_favorites_news(true, $plugin_cookies);
        } else {
            $currentCategoryNews = $this->api->getCategoryNews($channel_id);
        }
        
        
        // if clicked in daily EPG
        if($archive_ts > 0) {
            $indexByTimestam = $this->api->getIndexByTimestamp($channel_id, $archive_ts);
            // return button tries to rewind back 
            if($indexByTimestam > 0) {
                $this->current_news_item_index = $indexByTimestam;
                //hd_print("======= get_tv_playback_url, getIndexByTimestamp=$this->current_news_item_index");
            }
        }
        $media_str = null; $user_data = null;
        HD::decode_user_data($currentCategoryNews[$this->current_news_item_index][PluginRegularFolderItem::media_url],$media_str, $user_data);
        
        // play next
        $this->current_news_item_index++;
        if(count($currentCategoryNews) == $this->current_news_item_index) {
            // last news item, start playing from the first
            $this->current_news_item_index = 0;
        }
        
        //hd_print("======= get_tv_playback_url, url=$media_str, next_news_item_index=$this->current_news_item_index");
        return $media_str;
    }

    ///////////////////////////////////////////////////////////////////////
    // List<PluginTvEpgProgram>
    // return Electronic program guide list for a channel and day that starts on $day_start_tm_sec
    public function get_day_epg($channel_id, $day_start_tm_sec, &$plugin_cookies) {
        //hd_print("======= get_day_epg=$channel_id, $day_start_tm_sec");
        $dayEpg = array();
        if( $channel_id == _24tvApi::$favorites_url ) {
            $currentCategoryNews = $this->get_favorites_news(true, $plugin_cookies);
            for($i = 0; $i < count($currentCategoryNews); $i++) {
                $newsItem = $currentCategoryNews[$i];
                $isInLocalDay = HD::isInLocalDay($newsItem['timestamp'], $day_start_tm_sec);
                if ($isInLocalDay == 0) {
                    $dayEpg[] = array (
                        PluginTvEpgProgram::start_tm_sec => $newsItem['timestamp'],
                        PluginTvEpgProgram::end_tm_sec => $newsItem['timestamp'],
                        PluginTvEpgProgram::name => $newsItem[PluginRegularFolderItem::caption],
                        PluginTvEpgProgram::description => ''
                    );
                    
                } else if ($isInLocalDay > 0) {
                    //hd_print("======= get_day_epg, break, for timestamp=" . $newsItem['timestamp']);
                    // break;
                }
                $isInLocalDay = HD::isInLocalDay($currentCategoryNews[$i]['timestamp'], $day_start_tm_sec);
            }
        } else {
            $currentCategoryNews = $this->api->getCategoryNews($channel_id);
            // RSS items are oreder by timestamp descending, reverse ordering
            for($i = count($currentCategoryNews) - 1; $i > -1; $i--) {
                $newsItem = $currentCategoryNews[$i];
                $isInLocalDay = HD::isInLocalDay($newsItem['timestamp'], $day_start_tm_sec);
                if ($isInLocalDay == 0) {
                    $dayEpg[] = array (
                        
                        PluginTvEpgProgram::start_tm_sec => $newsItem['timestamp'],
                        PluginTvEpgProgram::end_tm_sec => $newsItem['timestamp'],
                        //PluginTvEpgProgram::end_tm_sec => -1,
                        PluginTvEpgProgram::name => $newsItem[PluginRegularFolderItem::caption],
                        PluginTvEpgProgram::description => ''
                    );
                } else if ($isInLocalDay > 0) {
                    //hd_print("======= get_day_epg, break, for timestamp=" . $newsItem['timestamp']);
                    break;
                }
                $isInLocalDay = HD::isInLocalDay($currentCategoryNews[$i]['timestamp'], $day_start_tm_sec);
            }
        }
        return $dayEpg;
        /*
        $date = DateTime::createFromFormat('H:i:s d.m.y', '23:59:59 04.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);        
        hd_print("======= get_day_epg, 23:59:59=$isIn");

        $date = DateTime::createFromFormat('H:i:s d.m.y', '00:00:00 05.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);        
        hd_print("======= get_day_epg, 00:00:00=$isIn");
        
        $date = DateTime::createFromFormat('H:i:s d.m.y', '01:59:59 05.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);        
        hd_print("======= get_day_epg, 1:59:59=$isIn");

        $date = DateTime::createFromFormat('H:i:s d.m.y', '3:00:00 05.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);        
        hd_print("======= get_day_epg, 3:00:00=$isIn");

        $date = DateTime::createFromFormat('H:i:s d.m.y', '03:00:01 05.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);
        hd_print("======= get_day_epg, 03:00:01=$isIn");

        $date = DateTime::createFromFormat('H:i:s d.m.y', '12:25:00 05.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);
        hd_print("======= get_day_epg, 12:25:00=$isIn");        

        $date = DateTime::createFromFormat('H:i:s d.m.y', '23:59:59 05.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);
        hd_print("======= get_day_epg, 23:59:59=$isIn");

        $date = DateTime::createFromFormat('H:i:s d.m.y', '00:00:00 06.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);
        hd_print("======= get_day_epg, 00:00:00=$isIn");

        $date = DateTime::createFromFormat('H:i:s d.m.y', '00:00:01 06.04.12', new DateTimeZone('Europe/Kiev'));
        $isIn = HD::isInLocalDay($date->getTimestamp(), $day_start_tm_sec,  $this->duneZoneOffset);
        hd_print("======= get_day_epg, 00:00:01=$isIn");
         */
    }

    ///////////////////////////////////////////////////////////////////////
    // main menu view
    private function get_main_folder_view($media_url, &$plugin_cookies) {
        //hd_print("======= get_main_folder_view, media_url=" . $media_url);

        return array (
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
            PluginFolderView::multiple_views_supported  => false,
            PluginFolderView::data                      => array
            (
                PluginRegularFolderView::async_icon_loading => false,
                
                PluginRegularFolderView::initial_range => $this->get_regular_folder_items($media_url, null, $plugin_cookies),

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 3,
                    ViewParams::num_rows => 3,
                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 370,
                    ViewParams::sandwich_height => 210,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => true,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_paint_caption => false,
                    ViewItemParams::icon_scale_factor => 0.85,
                    ViewItemParams::icon_sel_scale_factor => 1,
                    ViewItemParams::icon_path => 'gui_skin://small_icons/folder.aai',
                    ViewItemParams::item_layout => HALIGN_CENTER,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array (),
                PluginRegularFolderView::actions => array
                ( 
                    GUI_EVENT_KEY_ENTER => self::$open_folder_action,
                    GUI_EVENT_KEY_PLAY => self::$vod_play_action
                )
            )
        );
    }
    
    ///////////////////////////////////////////////////////////////////////
    // news list view
    private function get_category_folder_view($media_url, &$plugin_cookies) {
        //hd_print("======= get_categor_folder_view, media_url=" . $media_url);

        $view_params = array (
            ViewParams::num_cols => 1,
            ViewParams::num_rows => 12,
            ViewParams::paint_details => true,
            ViewParams::zoom_detailed_icon => true
        );
        $base_view_item_params = _24tvApi::$movie_list_vip;
        $view_params[ViewParams::paint_item_info_in_details] = true;
        $view_params[ViewParams::item_detailed_info_font_size] = FONT_SIZE_SMALL;
        $view_params[ViewParams::item_detailed_info_rel_y] = 315;
        $base_view_item_params[ViewItemParams::icon_path] = 'gui_skin://small_icons/movie.aai';

        if( $media_url === _24tvApi::$favorites_url) {
            $add_favorite_action = UserInputHandlerRegistry::create_action($this, 'remove_favorite');
            $add_favorite_action['caption'] = i18n::getText('btn.delete');
        } else {
            $add_favorite_action = UserInputHandlerRegistry::create_action($this, 'add_favorite');
            $add_favorite_action['caption'] = i18n::getText('btn.addFavorites');            
        }

        return array (
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
            PluginFolderView::multiple_views_supported  => false,
            PluginFolderView::data                      => array
            (
                PluginRegularFolderView::async_icon_loading => true,
                
                PluginRegularFolderView::initial_range => $this->get_regular_folder_items($media_url, null, $plugin_cookies),

                PluginRegularFolderView::view_params => $view_params,

                PluginRegularFolderView::base_view_item_params => $base_view_item_params,
                PluginRegularFolderView::not_loaded_view_item_params => array (),

                PluginRegularFolderView::actions => array
                (
                    GUI_EVENT_KEY_ENTER => self::$vod_play_action,
                    GUI_EVENT_KEY_PLAY => self::$vod_play_action,
                    GUI_EVENT_KEY_D_BLUE => $add_favorite_action
                )
            )
        );
    }
    
    ///////////////////////////////////////////////////////////////////////
    // root menu categories from RSS feed
    private function get_root_menu_folders() {
        $menus = $this->api->getRootMenuFolders();
        return array
        (
            PluginRegularFolderRange::total => count($menus),
            PluginRegularFolderRange::more_items_available => false,
            PluginRegularFolderRange::from_ndx => 0,
            PluginRegularFolderRange::count => count($menus),
            PluginRegularFolderRange::items => $menus
        );
    }

    ///////////////////////////////////////////////////////////////////////
    // news items from RSS feed
    private function get_category_news($media_url) {
        $news = $this->api->getCategoryNews($media_url);
        return array (
            PluginRegularFolderRange::total => count($news),
            PluginRegularFolderRange::more_items_available => false,
            PluginRegularFolderRange::from_ndx => 0,
            PluginRegularFolderRange::count => count($news),
            PluginRegularFolderRange::items => $news
        );
    }
    
    ///////////////////////////////////////////////////////////////////////
    private function get_favorites_news($epgFlg, &$plugin_cookies) {
        $news = array();
        
        // $this->api->set_fav_channel_ids($plugin_cookies, $news);

        $fav_channel_ids = array();
        $this->api->dailyEPGTimestampToIndex[_24tvApi::$favorites_url] = array();
        
        if (isset($plugin_cookies->{'favorite_channels'})) {
            $fav_channel_ids = preg_split("/"._24tvApi::$fav_separator."/", $plugin_cookies->{'favorite_channels'});
        }
        $cnt = count($fav_channel_ids);

        $news_item_index = 0;
        for($i=0; $i<$cnt; $i++) {
           
            $cookie_data = preg_split("/\|\|/", $fav_channel_ids[$i]);

            if(count($cookie_data) > 1) {
                $news_url = $cookie_data[0];
                $news_data = MediaURL::decode($cookie_data[1]);

                $vip = _24tvApi::$movie_list_vip;
                $vip[ViewItemParams::item_detailed_icon_path] = $news_data->thumbImage;
                $vip[ViewItemParams::icon_path] = 'gui_skin://small_icons/movie.aai';
                $vip[ViewItemParams::item_detailed_info] = i18n::getText('newsView.pubDate').' | '.HD::toLocalFormattedDateTime($news_data->timestamp);

                $news[] = array (
                        'timestamp' => $news_data->timestamp,
                        PluginRegularFolderItem::caption => $news_data->caption,
                        PluginRegularFolderItem::view_item_params => $vip,
                        PluginRegularFolderItem::media_url => 
                            HD::encode_user_data(
                                $news_url,
                                array ('channel_id' => _24tvApi::$favorites_url, 
                                       'news_item_index' => $news_data->news_item_index,
                                       'timestamp' => $news_data->timestamp,
                                       'caption' => str_replace("\"","'",$news_data->caption),
                                       'thumbImage' => $news_data->thumbImage
                                    )));
                $this->api->dailyEPGTimestampToIndex[_24tvApi::$favorites_url][$news_data->timestamp] = $news_item_index;
                $news_item_index++;
            }
        }
        if( $epgFlg == true ) {
            return $news;
        } else {
            return array (
                PluginRegularFolderRange::total => count($news),
                PluginRegularFolderRange::more_items_available => false,
                PluginRegularFolderRange::from_ndx => 0,
                PluginRegularFolderRange::count => count($news),
                PluginRegularFolderRange::items => $news
            );
        }
    }
    
    public function handle_user_input(&$user_input, &$plugin_cookies) {
        //hd_print('Tv favorites: handle_user_input:');
        //foreach ($user_input as $key => $value) {
            //hd_print("  $key => $value");
        //}
        
        
        if (!isset($user_input->selected_media_url)) {
            return null;
        }
        if ($user_input->control_id === 'add_favorite') {
            $is_favorite = $this->api->is_favorite_channel_id($user_input->selected_media_url, $plugin_cookies);
            if ($is_favorite) {
                return ActionFactory::show_title_dialog(i18n::getText('btn.favoritesAlreadyExists'));
            } else {
                $this->api->change_tv_favorites(PLUGIN_FAVORITES_OP_ADD, $user_input->selected_media_url, $plugin_cookies);
                return ActionFactory::show_title_dialog(i18n::getText('btn.addFavoritesSuccess'));
            }
        } else if ($user_input->control_id === 'remove_favorite') {
            $this->api->change_tv_favorites(PLUGIN_FAVORITES_OP_REMOVE, $user_input->selected_media_url, $plugin_cookies);
            return ActionFactory::show_title_dialog(i18n::getText('btn.deleteFavorites'), ActionFactory::invalidate_folders(array(_24tvApi::$favorites_url,)));
        }
        return null;
    }
    
}

?>