<?php
///////////////////////////////////////////////////////////////////////////

$currDuneDate = new DateTime();
HD::$duneZoneOffset = $currDuneDate->getOffset();

class HD {

    // seconds offset from UTC
    public static $duneZoneOffset;

    // format to be used for datetime
    private static $localDateTimeFormat = 'H:i d.m.y';

    ///////////////////////////////////////////////////////////////////////
    // the position as an integer. If needle is not found, strpos will return boolean false
    public static function starts_with($str, $pattern) {
        return strpos($str, $pattern) === 0;
    }

    ///////////////////////////////////////////////////////////////////////
    // encode url
    public static function encode_user_data($a, $b = null) {
        $media_url = null;
        $user_data = null;

        if (is_array ($a) && is_null($b))
        {
            $media_url = '';
            $user_data = $a;
        }
        else
        {
            $media_url = $a;
            $user_data = $b;
        }

        if (!is_null($user_data))
            // need to replace quotes with code
            $media_url .= '||' . json_encode(str_replace('"','\u0022',$user_data));

        return $media_url;
    }

    ///////////////////////////////////////////////////////////////////////
    // decode url
    public static function decode_user_data($media_url_str, &$media_url, &$user_data) {
        $idx = strpos($media_url_str, '||');

        if ($idx === false)
        {
            $media_url = $media_url_str;
            $user_data = null;
            return;
        }

        $media_url = substr($media_url_str, 0, $idx);
        $user_data = json_decode(substr($media_url_str, $idx + 2));
    }

    ///////////////////////////////////////////////////////////////////////
    // checks if the timestamp is in range of local date based on passed day start and timezone
    // params:
    //   $timestamp - unix time to check
    //   $day_start_tm_sec - unix day start
    //   $zoneOffset - seconds offset from UTC (local timezone offset)
    // return 0 if the timestamp belongs to a local day;
    // a value less than 0 if timestamp is before local day;
    // and a value greater than 0 if timestamp is after a local day.
    public static function isInLocalDay($timestamp, $day_start_tm_sec) {
        // 86400 - seconds in day
        return $day_start_tm_sec - self::$duneZoneOffset > $timestamp ? -1 :
            ($day_start_tm_sec - self::$duneZoneOffset + 86400 <= $timestamp ? 1 : 0);
    }

    ///////////////////////////////////////////////////////////////////////
    // represents passed timestamp as local date
    public static function toLocalFormattedDateTime($timestamp) {
        $localDate = new DateTime();
        $localDate->setTimestamp($timestamp);
        return $localDate->format(self::$localDateTimeFormat);
    }

}
///////////////////////////////////////////////////////////////////////////
?>
